@@ -185,7 +185,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
                 "negmas.sao.SAOMechanism": mechanisms.get(
                     "negmas.sao.SAOMechanism",
                     dict(
-                        end_on_no_response=False,
+                        end_on_no_response=True,
                         avoid_ultimatum=avoid_ultimatum,
                         dynamic_entry=False,
                         max_wait=len(agent_types),
@@ -1166,7 +1166,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         """
         current_balance = sum(self._profits[agent.id]) + self.initial_balances[agent.id]
         self.is_bankrupt[agent.id] = (
-            current_balance < self.bankruptcy_limit or self.is_bankrupt[agent.id]
+            (current_balance < self.bankruptcy_limit) or self.is_bankrupt[agent.id]
         )
         report = FinancialReport(
             agent_id=agent.id,
@@ -1299,6 +1299,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             + self._sold_quantity[has_trade, s + 1]
         )
         self._trading_price[has_trade, s + 1] /= self._betas_sum[has_trade, s + 1]
+        self._trading_price[:, s + 1:] = self._trading_price[:, s + 1].reshape((self.n_products, 1))
         self._traded_quantity += self._sold_quantity[:, s + 1]
         # self._trading_price[has_trade, s] = (
         #         np.sum(self._betas[:s+1] * self._real_price[has_trade, s::-1])
@@ -1672,6 +1673,8 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             `True` if the partner accepted and the negotiation is ready to start
 
         """
+        if self.is_bankrupt[agent_id]:
+            return True
         if controller is not None and negotiators is not None:
             raise ValueError(
                 "You cannot pass both controller and negotiators to request_negotiations"
