@@ -47,6 +47,7 @@ class AWI(AgentWorldInterface):
           - *inputs*: Inputs to every manufacturing process.
           - *outputs*: Outputs to every manufacturing process.
           - *is_system*: Is the given system ID corresponding to a system agent?
+          - *is_bankrupt*: Is the given agent bankrupt (None asks about self)?
           - *current_step*: Current simulation step (inherited from `negmas.situated.AgentWorldInterface` ).
           - *n_steps*: Number of simulation steps (inherited from `negmas.situated.AgentWorldInterface` ).
           - *relative_time*: fraction of the simulation completed (inherited from `negmas.situated.AgentWorldInterface`).
@@ -117,18 +118,9 @@ class AWI(AgentWorldInterface):
             is at level n_products - 1, this will be empty.
 
         D. Agent Information:
-          - *current_exogenous_input_quantity*: The total quantity the agent have
-            in its input exogenous contract.
-          - *current_exogenous_input_price*: The total price of the agent's
-            input exogenous contract.
-          - *current_exogenous_output_quantity*: The total quantity the agent have
-            in its output exogenous contract.
-          - *current_exogenous_output_price*: The total price of the agent's
-            output exogenous contract.
-          - *current_disposal_cost*: The disposal cost per unit item in the current
-            step.
-          - *current_shortfall_penalty*: The shortfall penalty per unit item in the current
-            step.
+          - *spot_market_quantity*: The quantity the agent bought from the spot market at
+                                    a given step
+          - *spot_market_loss*: The spot market loss for the agent.
 
     Actions:
         A. Negotiation Control:
@@ -204,6 +196,8 @@ class AWI(AgentWorldInterface):
 
 
         """
+        if self.is_bankrupt():
+            return False
         if controller is not None and negotiators is not None:
             raise ValueError(
                 "You cannot pass both controller and negotiators to request_negotiations"
@@ -246,7 +240,7 @@ class AWI(AgentWorldInterface):
             self.request_negotiation(
                 is_buy, product, quantity, unit_price, time, partner, negotiator, extra
             )
-            if not self._world.a2f[partner].is_bankrupt
+            if not self.is_bankrupt(partner)
             and self._world.can_negotiate(partner, self.agent.id)
             else False
             for partner, negotiator in zip(partners, negotiators)
@@ -303,7 +297,9 @@ class AWI(AgentWorldInterface):
 
 
         """
-        if self._world.a2f[partner].is_bankrupt or not self._world.can_negotiate(
+        if self.is_bankrupt():
+            return False
+        if self.is_bankrupt(partner) or not self._world.can_negotiate(
             partner, self.agent.id
         ):
             return False
@@ -693,3 +689,42 @@ class AWI(AgentWorldInterface):
             aid: Agent ID
         """
         return is_system_agent(aid)
+
+    def is_bankrupt(self, aid: Optional[str] = None) -> bool:
+        """
+        Checks whether the agent is bankrupt
+
+        Args:
+            aid: Agent ID (None means self)
+        """
+        if not aid:
+            aid = self.agent.id
+        return self._world.a2f[aid].is_bankrupt
+
+    def spot_market_quantity(self, step: Optional[int]) -> int:
+        """
+        The quantity bought by the agent from the spot market at the given step.
+
+        Args:
+            step: The simulation step (day)
+
+        Remarks:
+            If step is `None`, the current step will be used
+        """
+        if step is None:
+            step = self.current_step
+        return self._world._spot_quantity[self._world.a2i[self.agent.id], step]
+
+    def spot_market_loss(self, step: Optional[int]) -> int:
+        """
+        The spot market loss of the agent at the given step.
+
+        Args:
+            step: The simulation step (day)
+
+        Remarks:
+            If step is `None`, the current step will be used
+        """
+        if step is None:
+            step = self.current_step
+        return self._world._agent_spot_loss[self._world.a2i[self.agent.id], step]
