@@ -4,40 +4,43 @@ from os import PathLike
 from random import randint
 from random import random
 from random import shuffle
+from typing import Optional
+from typing import Tuple
 
 import numpy as np
 from negmas import Agent
 from negmas.helpers import get_full_type_name
+from negmas.serialization import serialize, deserialize
 from negmas.helpers import unique_name
 from negmas.tournaments import TournamentResults
 from negmas.tournaments import WorldRunResults
 from negmas.tournaments import tournament
+from scipy.stats import tmean
 
+from scml.oneshot.agents import RandomOneShotAgent
+from scml.oneshot.agents import SyncRandomOneShotAgent
 from scml.oneshot.world import SCML2020OneShotWorld
 from scml.scml2020.agents import BuyCheapSellExpensiveAgent
-from scml.scml2020.agents import (
-    DecentralizingAgent,
-    MarketAwareDecentralizingAgent,
-    MarketAwareIndDecentralizingAgent,
-    MarketAwareMovingRangeAgent,
-)
-from scml.scml2020.world import SCML2020World
+from scml.scml2020.agents import DecentralizingAgent
+from scml.scml2020.agents import MarketAwareDecentralizingAgent
+from scml.scml2020.agents import MarketAwareIndDecentralizingAgent
+from scml.scml2020.agents import SatisficerAgent
+from scml.scml2020.agents import MarketAwareMovingRangeAgent
+from scml.scml2020.world import SCML2020World, SCML2021World
 from scml.scml2020.world import is_system_agent
-from scml.oneshot.agents import RandomOneShotAgent, SyncRandomOneShotAgent
 
 if True:
-    from typing import (
-        Tuple,
-        Union,
-        Type,
-        Iterable,
-        Sequence,
-        Optional,
-        Callable,
-        Any,
-        Dict,
-        List,
-    )
+    from typing import Any
+    from typing import Callable
+    from typing import Dict
+    from typing import Iterable
+    from typing import List
+    from typing import Optional
+    from typing import Sequence
+    from typing import Tuple
+    from typing import Type
+    from typing import Union
+
     from .world import SCML2020Agent
 
 __all__ = [
@@ -56,6 +59,9 @@ __all__ = [
 ]
 
 
+FORCED_LOGS_FRACTION = 1.0
+
+
 ROUND_ROBIN = True
 
 
@@ -64,15 +70,16 @@ DefaultAgents = [DecentralizingAgent, BuyCheapSellExpensiveAgent]
 
 DefaultAgents2021 = [
     DecentralizingAgent,
-    MarketAwareDecentralizingAgent,
+    # MarketAwareDecentralizingAgent,
     MarketAwareIndDecentralizingAgent,
-    RandomOneShotAgent,
+    SatisficerAgent,
+    # RandomOneShotAgent,
 ]
 
 
 DefaultAgentsOneShot = [
     RandomOneShotAgent,
-    SyncRandomOneShotAgent,
+    # SyncRandomOneShotAgent,
 ]
 
 
@@ -205,21 +212,20 @@ def anac2020_config_generator(
     agent_names_reveal_type: bool = False,
     non_competitors: Optional[Tuple[Union[str, SCML2020Agent]]] = None,
     non_competitor_params: Optional[Tuple[Dict[str, Any]]] = None,
-    compact: bool = True,
+    compact: bool = False,
     *,
     n_steps: Union[int, Tuple[int, int]] = (50, 200),
     n_processes: Tuple[int, int] = (
-        3,
-        5,
+        2,
+        4,
     ),  # minimum is strictly guarantee but maximum is only guaranteed if select_n_levels_first
     min_factories_per_level: int = 2,  # strictly guaranteed
     max_factories_per_level: int = 6,  # not strictly guaranteed except if select_n_levels_first is False
     n_lines: int = 10,
-    select_n_levels_first=False,
+    select_n_levels_first=True,
     oneshot_world: bool = False,
     **kwargs,
 ) -> List[Dict[str, Any]]:
-
     if non_competitors is None:
         non_competitors = DefaultAgents
         non_competitor_params = [dict() for _ in non_competitors]
@@ -342,6 +348,26 @@ def anac2020_config_generator(
             no_logs=no_logs,
         )
     world_params.update(kwargs)
+    # _agent_types = copy.deepcopy(world_params.pop("agent_types"))
+    # _agent_params = copy.deepcopy(world_params.pop("agent_params"))
+    if oneshot_world:
+        generated_world_params = SCML2020OneShotWorld.generate(**world_params)
+    else:
+        generated_world_params = SCML2021World.generate(**world_params)
+    # world_params["agent_types"] = _agent_types
+    # world_params["agent_params"] = _agent_params
+    for k in ("agent_types", "agent_params"):
+        if k in generated_world_params.keys():
+            del generated_world_params[k]
+    if oneshot_world:
+        for _p in generated_world_params["profiles"]:
+            _p.cost = int(_p.cost)
+    else:
+        for _p in generated_world_params["profiles"]:
+            _p.costs = _p.costs.tolist()
+    world_params["__exact_params"] = serialize(
+        generated_world_params, deep=True, ignore_lambda=True
+    )
     config = {
         "world_params": world_params,
         "compact": compact,
@@ -392,11 +418,12 @@ def anac2020_assigner(
         # removing the competitors from the dynamic competitors
         if exclude_competitors_from_reassignment:
             # TODO May be use a better way to hash the a parameters than just conversion to str
-            compset = set(zip(competitors, (str(_) for _ in params)))
+            # Note that None and and empty dict() will both become ""
+            compset = set(zip(competitors, (str(_) if _ else "" for _ in params)))
             dynset = list(
                 zip(
                     dynamic_non_competitors,
-                    (str(_) for _ in dynamic_non_competitor_params),
+                    (str(_) if _ else "" for _ in dynamic_non_competitor_params),
                 )
             )
             dynamic_non_competitor_indices = [
@@ -480,11 +507,17 @@ def anac2020_world_generator(**kwargs):
     assert sum(kwargs["world_params"]["n_agents_per_process"]) == len(
         kwargs["world_params"]["agent_types"]
     )
-    cnfg = SCML2020World.generate(**kwargs["world_params"])
+    cnfg = kwargs["world_params"].pop("__exact_params")
+    cnfg = deserialize(cnfg)
+    cnfg2 = SCML2021World.generate(**kwargs["world_params"])
+    for k in ("agent_types", "agent_params"):
+        cnfg[k] = cnfg2[k]
+    for _p in cnfg["profiles"]:
+        _p.costs = np.asarray(_p.costs)
     if "info" not in cnfg.keys():
         cnfg["info"] = dict()
     cnfg["info"]["is_default"] = kwargs["is_default"]
-    world = SCML2020World(**cnfg)
+    world = SCML2021World(**cnfg)
     return world
 
 
@@ -492,8 +525,14 @@ def anac2020oneshot_world_generator(**kwargs):
     assert sum(kwargs["world_params"]["n_agents_per_process"]) == len(
         kwargs["world_params"]["agent_types"]
     )
-    # breakpoint()
-    cnfg = SCML2020OneShotWorld.generate(**kwargs["world_params"])
+    # cnfg = SCML2020OneShotWorld.generate(**kwargs["world_params"])
+    # for k in ("n_agents_per_process","n_processes"):
+    #     del kwargs["world_params"][k]
+    cnfg = kwargs["world_params"].pop("__exact_params")
+    cnfg = deserialize(cnfg)
+    cnfg2 = SCML2020OneShotWorld.generate(**kwargs["world_params"])
+    for k in ("agent_types", "agent_params", "name"):
+        cnfg[k] = cnfg2[k]
     if "info" not in cnfg.keys():
         cnfg["info"] = dict()
     cnfg["info"]["is_default"] = kwargs["is_default"]
@@ -841,6 +880,7 @@ def anac2020_std(
     configs_only=False,
     compact=False,
     n_competitors_per_world=None,
+    forced_logs_fraction: float = FORCED_LOGS_FRACTION,
     **kwargs,
 ) -> Union[TournamentResults, PathLike]:
     """
@@ -882,6 +922,7 @@ def anac2020_std(
         compact: If true, compact logs will be created and effort will be made to reduce the memory footprint
         n_competitors_per_world: Number of competitors in every simulation. If not given it will be a random number
                                  between 2 and min(2, n), where n is the number of competitors
+        forced_logs_fraction: Fraction of simulations for which logs are always saved (including negotiations)
         kwargs: Arguments to pass to the `world_generator` function
 
     Returns:
@@ -935,7 +976,7 @@ def anac2020_std(
         dynamic_non_competitor_params=dynamic_non_competitor_params,
         exclude_competitors_from_reassignment=exclude_competitors_from_reassignment,
         save_video_fraction=0.0,
-        forced_logs_fraction=0.0,
+        forced_logs_fraction=forced_logs_fraction,
         **kwargs,
     )
 
@@ -966,6 +1007,7 @@ def anac2020_collusion(
     configs_only=False,
     compact=False,
     n_competitors_per_world=None,
+    forced_logs_fraction: float = FORCED_LOGS_FRACTION,
     **kwargs,
 ) -> Union[TournamentResults, PathLike]:
     """
@@ -1008,6 +1050,7 @@ def anac2020_collusion(
         verbose: Verbosity
         configs_only: If true, a config file for each
         compact: If true, compact logs will be created and effort will be made to reduce the memory footprint
+        forced_logs_fraction: Fraction of simulations for which logs are always saved (including negotiations)
         kwargs: Arguments to pass to the `world_generator` function
 
     Returns:
@@ -1061,11 +1104,115 @@ def anac2020_collusion(
         dynamic_non_competitor_params=dynamic_non_competitor_params,
         exclude_competitors_from_reassignment=exclude_competitors_from_reassignment,
         save_video_fraction=0.0,
-        forced_logs_fraction=0.0,
+        forced_logs_fraction=forced_logs_fraction,
         **kwargs,
     )
 
 
+def truncated_mean(
+    scores: np.ndarray,
+    limits: Optional[Tuple[float, float]] = None,
+    top_limit=2.0,
+    bottom_limit=float("inf"),
+    base="tukey",
+    return_limits=False,
+) -> Union[float, Tuple[float, Optional[Tuple[float, float]]]]:
+    """
+    Calculates the truncated mean
+
+    Args:
+        scores: A list of scores for which to calculate the truncated mean
+        limits: The limits to use for trimming the scores. If not given, they will
+                be calculated based on `top_fraction`, `bottom_fraction` and `base.`
+                You can pass the special value "mean" as a string to disable limits and
+                calcualte the mean. You can pass the special value "median" to calculate
+                the median (which is the same as passing top_fraction==bottom_fraction=0.5
+                and base == "scores").
+        top_limit: top limit on scores to use for truncated mean calculation. See `base`
+        bottom_limit: bottom limit on scores to use for truncated mean calculation. See `base`
+        base: The base for calculating the limits used to apply the `top_limit` and `bottom_limit`.
+              Possible values are:
+              - zscore: the number of sigmas to remove above/below. A good default choice is 3. Pass inf to disable a side.
+              - tukey: the fraction of IQR to remove above/below. A good default choice is 1.5 or 3 (we use 2). Pass inf to disable a side.
+              - iqr : same as tukey
+              - iqr_fraction: the fraction is interpreted as the fraction of scores above/below the 1st/3rd qauntile
+              - scores: the fraction is interpreted as fraction of highest and lowest scores
+              - fraction: the fraction is interpreted as literal fraction of the values (i.e. given 10 values and 0.1, removes 1 value)
+              - mean: simply returns the mean (limits ignored)
+              - median: simply returns the median (limits ignored)
+        return_limits: If true, the method will also return the limiting scores used in its mean calculation.
+    """
+
+    scores = np.asarray(scores)
+    scores = scores[~np.isnan(scores)]
+
+    if isinstance(limits, str) and limits.lower() == "mean":
+        return tmean(scores, None) if not return_limits else (tmean(scores, None), None)
+    if isinstance(limits, str) and limits.lower() == "median":
+        return np.median(scores) if not return_limits else (np.median(scores), None)
+    if limits is not None:
+        return np.mean(scores) if not return_limits else (np.mean(scores), None)
+
+    if base == "zscore":
+        m, s = np.nanmean(scores), np.nanstd(scores)
+        limits = (m - s * bottom_limit, m + s * top_limit)
+    elif base in ("tukey", "iqr"):
+        q1, q3 = np.quantile(scores, 0.25), np.quantile(scores, 0.75)
+        iqr = q3 - q1
+        limits = (
+            q1 - (bottom_limit * iqr if not np.isinf(bottom_limit) else bottom_limit),
+            q3 + (top_limit * iqr if not np.isinf(top_limit) else top_limit),
+        )
+    elif base == "iqr_fraction":
+        bottom_limit = min(1, max(0, bottom_limit))
+        top_limit = min(1, max(0, top_limit))
+        limits = (np.quantile(scores, 0.25), np.quantile(scores, 0.75))
+        high = np.sort(scores[scores > limits[1]])
+        low = np.sort(scores[scores < limits[0]])
+        limits = (
+            low[int((len(low) - 1) * bottom_limit)] if len(low) > 0 else None,
+            high[int((len(high) - 1) * (1 - top_limit))] if len(high) > 0 else None,
+        )
+    elif base == "fraction":
+        bottom_limit = min(1, max(0, bottom_limit))
+        top_limit = min(1, max(0, top_limit))
+        scores = np.sort(scores)
+        top_indx = int((len(scores) - 1) * (1 - top_limit))
+        bottom_indx = int((len(scores) - 1) * bottom_limit)
+        if top_indx < bottom_indx:
+            return float("nan") if not return_limits else (float("nan"), limits)
+        m = np.mean(scores[bottom_indx : top_indx + 1])
+        return m if not return_limits else (m, (scores[bottom_indx], scores[top_indx]))
+    elif base == "scores":
+        bottom_limit = min(1, max(0, bottom_limit))
+        top_limit = min(1, max(0, top_limit))
+        limits = (
+            np.quantile(scores, bottom_limit),
+            np.quantile(scores, 1 - top_limit),
+        )
+        if limits[0] > limits[1]:
+            return float("nan") if not return_limits else (float("nan"), limits)
+    elif base == "mean":
+        return np.mean(scores) if not return_limits else (np.mean(scores), None)
+    elif base == "median":
+        return np.median(scores) if not return_limits else (np.median(scores), None)
+    else:
+        raise ValueError(f"Unknown base for truncated_mean ({base})")
+    if len(scores) == 0 or limits[1] < limits[0]:
+        return float("nan") if not return_limits else (float("nan"), limits)
+    try:
+        # this is an inclusive trimmed mean
+        # tm = tmean(scores, limits)
+        scores = scores[scores >= limits[0]]
+        scores = scores[scores <= limits[1]]
+        if len(scores) == 0:
+            return float("nan") if not return_limits else (float("nan"), limits)
+        tm = np.mean(scores)
+        return tm if not return_limits else (tm, limits)
+    except ValueError:
+        return float("nan") if not return_limits else (float("nan"), limits)
+
+
 def anac2021_tournament(
     competitors: Sequence[Union[str, Type[SCML2020Agent]]],
     agent_names_reveal_type=False,
@@ -1178,6 +1325,7 @@ def anac2021_std(
     configs_only=False,
     compact=False,
     n_competitors_per_world=None,
+    forced_logs_fraction: float = FORCED_LOGS_FRACTION,
     **kwargs,
 ) -> Union[TournamentResults, PathLike]:
     """
@@ -1219,6 +1367,7 @@ def anac2021_std(
         compact: If true, compact logs will be created and effort will be made to reduce the memory footprint
         n_competitors_per_world: Number of competitors in every simulation. If not given it will be a random number
                                  between 2 and min(2, n), where n is the number of competitors
+        forced_logs_fraction: Fraction of simulations for which logs are always saved (including negotiations)
         kwargs: Arguments to pass to the `world_generator` function
 
     Returns:
@@ -1266,13 +1415,13 @@ def anac2021_std(
         score_calculator=balance_calculator2021,
         min_factories_per_level=min_factories_per_level,
         compact=compact,
-        metric="median",
+        metric=truncated_mean,
         n_competitors_per_world=n_competitors_per_world,
         dynamic_non_competitors=dynamic_non_competitors,
         dynamic_non_competitor_params=dynamic_non_competitor_params,
         exclude_competitors_from_reassignment=exclude_competitors_from_reassignment,
         save_video_fraction=0.0,
-        forced_logs_fraction=0.0,
+        forced_logs_fraction=forced_logs_fraction,
         publish_exogenous_summary=True,
         publish_trading_prices=True,
         **kwargs,
@@ -1304,7 +1453,8 @@ def anac2021_collusion(
     verbose: bool = False,
     configs_only=False,
     compact=False,
-    n_competitors_per_world=None,
+    n_competitors_per_world=1,
+    forced_logs_fraction: float = FORCED_LOGS_FRACTION,
     **kwargs,
 ) -> Union[TournamentResults, PathLike]:
     """
@@ -1343,10 +1493,12 @@ def anac2021_collusion(
         dynamic_non_competitor_params: paramters of dynamic non competitor agents
         exclude_competitors_from_reassignment: If true, competitors are excluded from the dyanamic non-competitors
         n_competitors_per_world: Number of competitors in every simulation. If not given it will be a random number
-                                 between 2 and min(2, n), where n is the number of competitors
+                                 between 2 and min(2, n), where n is the number of competitors. This value will
+                                 always be set to 1 in SCML2021
         verbose: Verbosity
         configs_only: If true, a config file for each
         compact: If true, compact logs will be created and effort will be made to reduce the memory footprint
+        forced_logs_fraction: Fraction of simulations for which logs are always saved (including negotiations)
         kwargs: Arguments to pass to the `world_generator` function
 
     Returns:
@@ -1359,10 +1511,7 @@ def anac2021_collusion(
         processing
 
     """
-    if n_competitors_per_world is None:
-        n_competitors_per_world = kwargs.get(
-            "n_competitors_per_world", randint(2, min(4, len(competitors)))
-        )
+    n_competitors_per_world = 1
     kwargs.pop("n_competitors_per_world", None)
     if non_competitors is None:
         non_competitors = DefaultAgents
@@ -1394,13 +1543,13 @@ def anac2021_collusion(
         score_calculator=balance_calculator2021,
         min_factories_per_level=min_factories_per_level,
         compact=compact,
-        metric="median",
+        metric=truncated_mean,
         n_competitors_per_world=n_competitors_per_world,
         dynamic_non_competitors=dynamic_non_competitors,
         dynamic_non_competitor_params=dynamic_non_competitor_params,
         exclude_competitors_from_reassignment=exclude_competitors_from_reassignment,
         save_video_fraction=0.0,
-        forced_logs_fraction=0.0,
+        forced_logs_fraction=forced_logs_fraction,
         publish_exogenous_summary=True,
         publish_trading_prices=True,
         **kwargs,
@@ -1432,10 +1581,11 @@ def anac2021_oneshot(
     configs_only=False,
     compact=False,
     n_competitors_per_world=None,
+    forced_logs_fraction: float = FORCED_LOGS_FRACTION,
     **kwargs,
 ) -> Union[TournamentResults, PathLike]:
     """
-    The function used to run ANAC 2020 SCML tournament (standard track).
+    The function used to run ANAC 2021 SCML tournament (oneshot track).
 
     Args:
 
@@ -1473,6 +1623,7 @@ def anac2021_oneshot(
         compact: If true, compact logs will be created and effort will be made to reduce the memory footprint
         n_competitors_per_world: Number of competitors in every simulation. If not given it will be a random number
                                  between 2 and min(2, n), where n is the number of competitors
+        forced_logs_fraction: Fraction of simulations for which logs are always saved (including negotiations)
         kwargs: Arguments to pass to the `world_generator` function
 
     Returns:
@@ -1500,6 +1651,7 @@ def anac2021_oneshot(
         non_competitor_params = [dict() for _ in non_competitors]
     kwargs["round_robin"] = kwargs.get("round_robin", ROUND_ROBIN)
     kwargs["oneshot_world"] = True
+    kwargs["n_processes"] = 2
     return tournament(
         competitors=competitors,
         competitor_params=competitor_params,
@@ -1526,13 +1678,13 @@ def anac2021_oneshot(
         score_calculator=balance_calculator2021oneshot,
         min_factories_per_level=min_factories_per_level,
         compact=compact,
-        metric="median",
+        metric=truncated_mean,
         n_competitors_per_world=n_competitors_per_world,
         dynamic_non_competitors=dynamic_non_competitors,
         dynamic_non_competitor_params=dynamic_non_competitor_params,
         exclude_competitors_from_reassignment=exclude_competitors_from_reassignment,
         save_video_fraction=0.0,
-        forced_logs_fraction=0.0,
+        forced_logs_fraction=forced_logs_fraction,
         publish_exogenous_summary=True,
         publish_trading_prices=True,
         **kwargs,
