@@ -1,12 +1,17 @@
 import random
 from collections import defaultdict
+import numpy as np
+import pandas as pd
+from numpy.testing import assert_allclose
+from pytest import raises
 
 import hypothesis.strategies as st
 import pytest
 from hypothesis import given
 from hypothesis import settings
-from negmas import genius_bridge_is_running
+from negmas import genius_bridge_is_running, ResponseType
 from negmas import save_stats
+from negmas.sao import SAOResponse
 from negmas.genius import Atlas3
 from negmas.genius import GeniusNegotiator
 from negmas.genius import NiceTitForTat
@@ -20,8 +25,12 @@ from pytest import mark
 import scml
 from scml.oneshot import SCML2020OneShotWorld
 from scml.oneshot import builtin_agent_types
-from scml.oneshot.agent import OneShotIndNegotiatorsAgent
-from scml.oneshot.agents import RandomOneShotAgent
+from scml.oneshot.agent import (
+    OneShotIndNegotiatorsAgent,
+    OneShotSyncAgent,
+    OneShotAgent,
+)
+from scml.oneshot.agents import RandomOneShotAgent, GreedySyncAgent
 from scml.oneshot.common import QUANTITY
 from scml.oneshot.common import TIME
 from scml.oneshot.common import UNIT_PRICE
@@ -851,3 +860,334 @@ def test_ufun_penalty_scales_are_correct(penalties_scale):
                     u.input_penalty_scale
                     == a.awi.catalog_prices[a.awi.my_input_product]
                 )
+
+
+class MySyncAgent(OneShotSyncAgent):
+    new_step = False
+
+    def step(self):
+        self.new_step = True
+
+    def counter_all(self, offers, states):
+        """Respond to a set of offers given the negotiation state of each."""
+        if self.new_step:
+            raise RuntimeError("counter_all before first_proposals")
+        return dict(
+            zip(
+                self.negotiators.keys(),
+                [SAOResponse(ResponseType.END_NEGOTIATION, None)]
+                * len(self.negotiators),
+            )
+        )
+
+    def get_offer(self, negotiator_id: str):
+        ami = self.get_ami(negotiator_id)
+        quantity_issue = ami.issues[QUANTITY]
+        unit_price_issue = ami.issues[UNIT_PRICE]
+
+        offer = [-1] * 3
+        offer[QUANTITY] = quantity_issue.max_value
+        offer[TIME] = self.awi.current_step
+        offer[UNIT_PRICE] = unit_price_issue.max_value
+        return offer
+
+    def first_proposals(self):
+        """Decide a first proposal on every negotiation.
+        Returning None for a negotiation means ending it."""
+        self.new_step = False
+        return dict(
+            zip(
+                self.negotiators.keys(),
+                (self.get_offer(neg_id) for neg_id in self.negotiators.keys()),
+            )
+        )
+
+
+@mark.parametrize(
+    ("agent_types", "n_agents_per_process", "n_processes"),
+    [
+        ([], 2, 2),
+        ([], 3, 2),
+        ([], 4, 2),
+        ([], 5, 2),
+        ([], 6, 2),
+        ([RandomOneShotAgent], 2, 2),
+        ([RandomOneShotAgent], 3, 2),
+        ([RandomOneShotAgent], 4, 2),
+        ([RandomOneShotAgent], 5, 2),
+        ([RandomOneShotAgent], 6, 2),
+        ([GreedySyncAgent], 2, 2),
+        ([GreedySyncAgent], 3, 2),
+        ([GreedySyncAgent], 4, 2),
+        ([GreedySyncAgent], 5, 2),
+        ([GreedySyncAgent], 6, 2),
+    ],
+)
+def test_sync_agent_receives_first_proposals_before_counter_all(
+    agent_types, n_agents_per_process, n_processes
+):
+    n_steps = 50
+    world = SCML2020OneShotWorld(
+        **SCML2020OneShotWorld.generate(
+            [MySyncAgent] + agent_types,
+            n_processes=n_processes,
+            n_steps=n_steps,
+            n_lines=10,
+            n_agents_per_process=n_agents_per_process,
+            no_logs=True,
+            compact=True,
+            ignore_agent_exceptions=False,
+            ignore_negotiation_exceptions=False,
+            ignore_contract_execution_exceptions=False,
+            ignore_simulation_exceptions=False,
+        )
+    )
+    world.run()
+    assert world.current_step >= n_steps - 1
+
+
+class MyRandomAgent(RandomOneShotAgent):
+    def has_trade(self, s=None):
+        if s is None:
+            s = self.awi.current_step
+        return (self.awi._world._sold_quantity[:, s] > 0) & (
+            np.abs(self.awi._world._real_price[:, s] - self.catalog) >= 1
+        )
+
+    def trading_(self, s=None):
+        if s is None:
+            s = self.awi.current_step
+        return self.awi._world._trading_price[:, s]
+
+    def init(self):
+        super().init()
+        self.trading, self.trading_before = [], []
+        self.catalog = self.awi.catalog_prices.copy()
+        self.n_products = self.awi.n_products
+        assert_allclose(self.catalog, self.trading_())
+        assert_allclose(self.catalog, self.awi.trading_prices)
+
+    def before_step(self):
+        self.trading_before.append(self.awi.trading_prices.tolist())
+
+        assert_allclose(self.trading_before[-1], self.trading_())
+        assert_allclose(self.catalog, self.awi.catalog_prices)
+
+    def step(self):
+        super().step()
+        self.trading.append(self.awi.trading_prices.tolist())
+
+        assert_allclose(self.catalog, self.awi.catalog_prices)
+        assert_allclose(self.trading[-1], self.trading_())
+        with_trade = self.has_trade()
+
+        if not np.any(with_trade):
+            return
+        with raises(AssertionError):
+            assert_allclose(self.trading[-1], self.catalog)
+
+
+@mark.parametrize(
+    ["n_agents", "n_processes", "n_steps"],
+    [
+        (1, 2, 16),
+        (2, 2, 16),
+        (5, 2, 16),
+        (1, 3, 16),
+        (2, 3, 16),
+        (5, 3, 16),
+    ],
+)
+def test_trading_prices_updated(n_agents, n_processes, n_steps):
+    from scml.oneshot import SCML2020OneShotWorld
+    from negmas.helpers import force_single_thread
+
+    eps = 1e-3
+
+    world = SCML2020OneShotWorld(
+        **SCML2020OneShotWorld.generate(
+            [MyRandomAgent] * n_processes,
+            n_agents_per_process=n_agents,
+            n_processes=n_processes,
+            n_steps=n_steps,
+        ),
+        compact=True,
+        no_logs=True,
+    )
+    catalog_prices = world.catalog_prices
+    diffs = np.zeros_like(catalog_prices)
+
+    # we start at catlaog prices
+    for aid, agent in world.agents.items():
+        assert np.abs(agent.awi.trading_prices - catalog_prices).max() < eps
+
+    force_single_thread(True)
+    for _ in range(n_steps):
+        world.step()
+        trading_prices = None
+        for aid, agent in world.agents.items():
+            if is_system_agent(aid):
+                continue
+            trading_prices = agent.awi.trading_prices.copy()
+            break
+        diffs = np.maximum(diffs, np.abs(trading_prices - catalog_prices))
+
+    assert diffs.max() > eps
+    force_single_thread(False)
+
+
+class MyOneShotDoNothing(OneShotAgent):
+    def propose(self, negotiator_id, state):
+        return None
+
+    def respond(self, negotiator_id, state, offer):
+        return ResponseType.END_NEGOTIATION
+
+
+def test_do_nothing_goes_bankrupt():
+    world = generate_world([MyOneShotDoNothing], 2, 1000, 4, cash_availability=0.001)
+    world.run()
+    for aid, agent in world.agents.items():
+        if is_system_agent(aid):
+            continue
+        assert world.is_bankrupt[
+            aid
+        ], f"Agent {aid} is not bankrupt with balance {world.current_balance(aid)} and initial_balance of {world.initial_balances[aid]}"
+        assert agent.awi.is_bankrupt()
+
+
+class PricePumpingAgent(OneShotAgent):
+    """An agent that causes the intermediate price to go up over time"""
+
+    def top_outcome(self, negotiator_id):
+        return tuple(_.max_value for _ in self.get_ami(negotiator_id).issues)
+
+    def propose(self, negotiator_id, state):
+        return self.top_outcome(negotiator_id)
+
+    def respond(self, negotiator_id, state, offer):
+        return (
+            ResponseType.ACCEPT_OFFER
+            if self.top_outcome(negotiator_id) == offer
+            else ResponseType.REJECT_OFFER
+        )
+
+
+def check_trading_explosion(world, checked_types=(PricePumpingAgent,)):
+    stats = world.stats_df
+    contracts = pd.DataFrame(world.saved_contracts)
+    negotiations = pd.DataFrame(world.saved_negotiations)
+    for aid, agent in world.agents.items():
+        if is_system_agent(aid):
+            continue
+        if not agent.awi.is_last_level or not any(
+            issubclass(agent.__class__, _) for _ in checked_types
+        ):
+            continue
+        # all sellers should go bankrupt
+        assert world.is_bankrupt[aid]
+        assert agent.awi.is_bankrupt()
+        # bankrupt agents remain bankrupt
+        bankrupt = stats[f"bankrupt_{aid}"].values.astype(bool)
+        bankrupt_first_index = np.where(bankrupt == True)[0][0]
+        assert not np.any(bankrupt[:bankrupt_first_index])
+        assert np.all(bankrupt[bankrupt_first_index:])
+
+        # cannot become bankrupt without contracts
+        assert (
+            len(
+                contracts.loc[
+                    (contracts.signed_at <= bankrupt_first_index)
+                    & (contracts.buyer_name == aid),
+                    :,
+                ]
+            )
+            > 0
+        )
+
+        # no contracts after becoming bankrupt
+        assert (
+            len(
+                contracts.loc[
+                    (contracts.signed_at > bankrupt_first_index)
+                    & (contracts.buyer_name == aid),
+                    :,
+                ]
+            )
+            == 0
+        )
+
+        # no negotiations after becoming bankrupt
+        assert (
+            len(
+                negotiations.loc[
+                    (negotiations.requested_at > bankrupt_first_index)
+                    & (negotiations.partners.apply(lambda x: aid in x)),
+                    :,
+                ]
+            )
+            == 0
+        )
+
+        # some negotiations before becoming bankrupt
+        assert (
+            len(
+                negotiations.loc[
+                    (negotiations.requested_at <= bankrupt_first_index)
+                    & (negotiations.partners.apply(lambda x: aid in x)),
+                    :,
+                ]
+            )
+            > 0
+        )
+
+
+def test_price_pumping_happen():
+    world = generate_world([PricePumpingAgent], 2, 300, 4)
+    world.run()
+    check_trading_explosion(world)
+
+
+def test_price_pumping_happen_with_random_included():
+    world = generate_world([PricePumpingAgent, RandomOneShotAgent], 2, 300, 4)
+    world.run()
+    check_trading_explosion(world)
+
+
+def test_price_pumping_bankrupts_random_agents():
+    types = [PricePumpingAgent, RandomOneShotAgent]
+    world = generate_world(types, 2, 300, 4)
+    world.run()
+    check_trading_explosion(world, types)
+
+
+# class RationalRandomOneShotAgent(RandomOneShotAgent):
+#     def top_outcome(self, negotiator_id):
+#         return tuple(_.max_value for _ in self.get_ami(negotiator_id).issues)
+#
+#     def bottom_outcome(self, negotiator_id):
+#         return tuple(_.min_value for _ in self.get_ami(negotiator_id).issues)
+#
+#     def propose(self, negotiator_id, state):
+#         outcome = super().propose(negotiator_id, state)
+#         if self.awi.is_first_level and not self.ufun.ok_to_sell_at(outcome[UNIT_PRICE]):
+#             return self.top_outcome(negotiator_id)
+#         if self.awi.is_last_level and not self.ufun.ok_to_buy_at(outcome[UNIT_PRICE]):
+#             return self.bottom_outcome(negotiator_id)
+#         return outcome
+#
+#     def accept_if(self, x):
+#         return ResponseType.ACCEPT_OFFER if x else ResponseType.REJECT_OFFER
+#
+#     def respond(self, negotiator_id, state, offer):
+#         if self.awi.is_first_level:
+#             return self.accept_if(self.ufun.ok_to_sell_at(offer[UNIT_PRICE]))
+#         if self.awi.is_last_level:
+#             return self.accept_if(self.ufun.ok_to_buy_at(offer[UNIT_PRICE]))
+#         return ResponseType.ACCEPT_OFFER
+#
+# def test_price_pumping_does_not_bankrupt_rational_random_agents():
+#     types = [PricePumpingAgent, RationalRandomOneShotAgent]
+#     world = generate_world(types, 2, 300, 4)
+#     world.run()
+#     check_trading_explosion(world, types)
